#ifndef _FLUID_FLUIDLISTENER_CPP_
#define _FLUID_FLUIDLISTENER_CPP_

#include <fluid.h>
#include <sx/Log4SX.h>
using namespace sx;

namespace fluid {

	ShadeX FluidListener::shadeX;

	unsigned int FluidListener::GROUPS = 160;
	const unsigned int FluidListener::RESPAWN_WIDTH = 5;
	const unsigned int FluidListener::RESPAWN_HEIGHT = 5;

	FluidListener::FluidListener() {
	}

	FluidListener::~FluidListener() {
	}

	Vector FluidListener::getMinRespawnPos() {
		float dx = 1.0f / (float)RESPAWN_WIDTH;
		float dy = 1.0f / (float)RESPAWN_HEIGHT;
		return Vector(dx * (float)respawn_pos_x, dy * (float)respawn_pos_y) * 2.0f + Vector(-1.0f,-1.0f);
	}

	Vector FluidListener::getRespawnArea() {
		return Vector(1.0f / (float)RESPAWN_WIDTH , 1.0f / (float)RESPAWN_HEIGHT) * 2.0f;
	}

	void FluidListener::setSolidMap(const string &path) {
		Texture &t = shadeX.getTexture("solid.texture");
		t.setPath(path);
		t.load();
		if(!t.isLoaded()) {
			throw 1;
		}
	}
	
	void FluidListener::setBackgroundMap(const string &path) {
		Texture &t = shadeX.getTexture("initfluid.texture");
		t.setPath(path);
		t.load();
		stopDrawTime = 0;
		drawTime = 1;
		if(!t.isLoaded()) {
			throw 1;
		}
	}

	void FluidListener::create(SXRenderArea &area) {
		this->pressed = false;

		try {
			shadeX.addResources("../data/configuration/fluid.c");
			shadeX.addResources("../data/configuration/glyph.c");
			shadeX.load();
		} catch(Exception &e) {
			Logger::get() << Level(L_FATAL_ERROR) << e.getMessage();
			area.stopRendering();
			return;
		}

		drawTime = 1;
		stopDrawTime = 15.0f;
		drawOrigin = &shadeX.getUniformVector("draw.origin.vector");
		drawOrigin->random();

		view = VIEW_STANDARD;

		updateGlyphCount = false;

		respawn_pos_x = 0;
		respawn_pos_y = 0;

		grid = new StreamletGrid(22,22);

		for(unsigned int i=0 ; i<GROUPS ; i++) {
			Glyph *glyph = new Glyph(i);
			glyphs.push_back(glyph);
			ready.push_back(glyph);
		}

		Effect &initEffect = shadeX.getEffect("initfluid.effect");
		initEffect.render();
	}

	void FluidListener::reshape(SXRenderArea &area) {
		RenderTarget &presentTarget = shadeX.getRenderTarget("present.rendertarget");
		presentTarget.setWidth(area.getWidth());
		presentTarget.setHeight(area.getHeight());
		presentTarget.setRenderToDisplay(false);
		presentTarget.load();

		RenderTarget &shadowtarget = shadeX.getRenderTarget("present.offline.target");
		shadowtarget.setWidth(area.getWidth());
		shadowtarget.setHeight(area.getHeight());
		shadowtarget.setRenderToDisplay(false);
		shadowtarget.load();

		Texture &shadowmap = shadeX.getTexture("glyphshadow.texture");
		shadowmap.setWidth(area.getWidth());
		shadowmap.setHeight(area.getHeight());
		shadowmap.load();

		Texture &shadowmap2 = shadeX.getTexture("glyphshadow2.texture");
		shadowmap2.setWidth(area.getWidth());
		shadowmap2.setHeight(area.getHeight());
		shadowmap2.load();
	}

	void FluidListener::render(SXRenderArea &area) {
		if(updateGlyphCount) {
			//update with new glyph count
			updateGlyphCount = false;

			//deconstruct previously initialized glyphs
			ready.clear();
			dead.clear();
			alive.clear();
			for(list<Glyph *>::iterator iter = glyphs.begin() ; iter != glyphs.end() ; iter++) {
				delete (*iter);
			}
			glyphs.clear();

			//start with a new set of glyphs
			for(unsigned int i=0 ; i<GROUPS ; i++) {
				Glyph *glyph = new Glyph(i);
				glyphs.push_back(glyph);
				ready.push_back(glyph);
			}
		}

		//initialize all controlpoints of the arrow
		//with its midpoint (= handle point)

		UniformFloat &ugroupsize = shadeX.getUniformFloat("groupsize.count");
		unsigned int groupsize = (unsigned int)ugroupsize.value;
		unsigned int midpointpos = (unsigned int)floor((float)groupsize/2.0f);
		BufferedMesh &posmesh = shadeX.getBufferedMesh("positions1.mesh");
		VertexBuffer *posbuf = posmesh.getBuffer("pos");
		if(posbuf != 0) {
			float *points = posbuf->unlockWrite();

			list<Glyph *>::iterator glyphIter = glyphs.begin();
			for(unsigned int group=0 ; group<GROUPS ; group++) {
				Glyph *glyph = (*glyphIter);

				const Vector &midpoint = glyph->getMidpoint();
				for(unsigned int e=0 ; e<groupsize ; e++) {
					if(e == midpointpos) {
						continue;
					}
					for(unsigned int i=0 ; i<3 ; i++) {
						points[group*groupsize*4 + e*4 + i] = midpoint[i];
					}
					points[group*groupsize*4 + e*4 + 3] = ((float)e) - ((float)midpointpos);
				}

				glyphIter++;
			}

			posbuf->lock();
		}

		//random gravity

		Vector &gravity = shadeX.getUniformVector("gravity.vector");
		gravity = 0.1f * 2.0f * (Vector().random() + Vector(-0.5f,-0.5f,-0.5f));

		//do random drawing
		if(stopDrawTime < 0) {
			if(drawTime >= 0) {
				*drawOrigin = Vector(0,0);
				Effect &draw = shadeX.getEffect("draw.effect");
				draw.render();

				drawTime -= (float)area.getDeltaTime();
			} else {
				drawTime = 1;
				stopDrawTime = 15;
				drawOrigin->random();
			}
		} else {
			stopDrawTime -= (float)area.getDeltaTime();
		}
		
		//handle input

		UniformFloat &D_time = shadeX.getUniformFloat("D_time.float");
		D_time.value = (float)area.getDeltaTime();

		if(area.hasMouseKey(MOUSE_LEFT)) {
			Vector &controlVelocity = shadeX.getUniformVector("control.velocity.vector");
			Vector &controlPosition = shadeX.getUniformVector("control.position.vector");

			controlVelocity = Vector();
			Vector newMousepos = Vector((float)area.getMouseX() / (float)area.getWidth(),(float)area.getMouseY() / (float)area.getHeight());
			if(pressed) {
				controlVelocity = (newMousepos + (-1.0f) * mousepos) * (1.0f/D_time.value);
				controlPosition = newMousepos;
				Effect &input = shadeX.getEffect("input.effect");
				input.render();
			}
			
			mousepos = newMousepos;
			pressed = true;
		} else {
			pressed = false;
		}

		UniformFloat &present = shadeX.getUniformFloat("present.float");
		if(view == VIEW_VORTICITY) {
			present.value = 0.0f;
		} else {
			present.value = 1.0f;
		}
	
		Vector &displacement = shadeX.getUniformVector("displacement.vector");
		displacement.random();
		displacement = 2.0f * (displacement + Vector(-0.5f,-0.5f));

		//do actual calculation

		Effect &calcEffect = shadeX.getEffect("calculate.effect");
		calcEffect.render();

		//upload arrow data to glyphs as uniforms

		if(posbuf != 0) {
			const float *points = posbuf->unlockRead();

			list<Glyph *>::iterator glyphIter = glyphs.begin();
			for(unsigned int group=0 ; group<GROUPS ; group++) {
				Glyph *glyph = (*glyphIter);

				glyph->updatePositions(&points[group*groupsize*4]);

				glyphIter++;
			}

			posbuf->lock();
		}

		BufferedMesh &velmesh = shadeX.getBufferedMesh("velocities.mesh");
		VertexBuffer *velbuf = velmesh.getBuffer("vel");
		if(velbuf != 0) {
			const float *vels = velbuf->unlockRead();

			list<Glyph *>::iterator glyphIter = glyphs.begin();
			for(unsigned int group=0 ; group<GROUPS ; group++) {
				Glyph *glyph = (*glyphIter);

				glyph->updateVelocities(&vels[group*groupsize*4]);

				glyphIter++;
			}

			velbuf->lock();
		}
		for(list<Glyph *>::iterator glyphIter = glyphs.begin() ; glyphIter != glyphs.end() ; glyphIter++) {
			(*glyphIter)->updateGlyphsize();
		}

		//do rendering

		Effect &presentEffect = shadeX.getEffect("present.effect");
		presentEffect.render();

		//update glyph states

		//put glyphs into grid
		grid->clear();
		for(list<Glyph *>::iterator iter = alive.begin() ; iter != alive.end() ; iter++) {
			Glyph *glyph = (*iter);
			grid->put(glyph);
		}
		grid->collide();

		if(rand() % 2 == 0) {
			//distribute glyphs from the ready list uniformly
			if(posbuf != 0) {
				float *points = posbuf->unlockWrite();

				unsigned int availableGlyphs = ready.size();
				unsigned int cells = RESPAWN_WIDTH * RESPAWN_HEIGHT;
				while(availableGlyphs > 0 && cells > 0) {
					//calculate random start position in cell
					Vector minPos = getMinRespawnPos();
					Vector respawnArea = getRespawnArea();
					Vector rvec = Vector().random();
					Vector startpos = minPos + Vector(respawnArea[0]*rvec[0],respawnArea[1]*rvec[1]);

					//test, if this position is still available
					//if that is the case, start with the current glyph
					//at startpos
					Glyph *glyph = ready.back();
					glyph->reset(startpos, &points[glyph->getID()*groupsize*4]);
					if(!grid->respawnCollide(glyph)) {
						ready.pop_back();
						availableGlyphs--;
						glyph->setAlive();
						alive.push_back(glyph);
					}

					//advance to next cell
					respawn_pos_x++;
					if(respawn_pos_x >= RESPAWN_WIDTH) {
						respawn_pos_x = 0;
						respawn_pos_y++;
						if(respawn_pos_y >= RESPAWN_HEIGHT) {
							respawn_pos_y = 0;
						}
					}
					cells--;
				}

				posbuf->lock();
			}
		} else {
			if(posbuf != 0) {
				float *points = posbuf->unlockWrite();

				//add glyphs close to glyphs with high vorticity
				//to emphasize those regions
				alive.sort(glyphComparison);
				list<Glyph *>::iterator iter = alive.begin();
				list<Glyph *> futureAlive;
				while(ready.size() > 0 && iter != alive.end()) {
					Glyph *seedGlyph = (*iter);

					//calculate line of a randomly choosen glyph segment
					int pos = rand() % seedGlyph->getGroupSize();
					int neighbourpos = (1 - 2*(rand() % 2)) + pos;
					if(neighbourpos < 0) {
						neighbourpos = pos + 1;
					} else if(neighbourpos >= (int)seedGlyph->getGroupSize()) {
						neighbourpos = pos - 1;
					}
					Vector line = (*seedGlyph)[pos] + (-1.0f) * (*seedGlyph)[neighbourpos];
					line[2] = 0;
					if(line.length() == 0) {
						line.random();
						line = line + Vector(0.01f,0.01f);
						line[2] = 0;
					}
					line.normalize();

					//generate seedpoint
					Vector normal(-line[1],line[0]);
					normal = normal * seedGlyph->respawnDistance();
					Vector seedpos = normal + (*seedGlyph)[pos];

					//initialize a ready glyph
					//at the seedpoint
					Glyph *glyph = ready.back();
					glyph->reset(seedpos, &points[glyph->getID()*groupsize*4]);
					if(!grid->respawnCollide(glyph)) {
						ready.pop_back();
						glyph->setAlive();
						futureAlive.push_back(glyph);
					}

					iter++;
				}
				for(list<Glyph *>::iterator iter=futureAlive.begin() ; iter!=futureAlive.end() ; iter++) {
					alive.push_back((*iter));
				}

				posbuf->lock();
			}
		}

		//search for dead arrows, put dead glyphs into the dead list
		for(list<Glyph *>::iterator iter = alive.begin() ; iter != alive.end() ; ) {
			Glyph *glyph = (*iter);
			if(glyph->getState() == STATE_DEAD) {
				//remove glyph from alive list, put glyph into the dead list
				alive.erase(iter++);
				dead.push_back(glyph);
			} else {
				iter++;
			}
		}

		//search for ready arrows, put ready glyphs from the dead into the ready list
		for(list<Glyph *>::iterator iter = dead.begin() ; iter != dead.end() ; ) {
			Glyph *glyph = (*iter);
			if(glyph->getState() != STATE_DEAD) {
				dead.erase(iter++);
				ready.push_back(glyph);
			} else {
				iter++;
			}
		}

		//update glyph state switch time
		for(list<Glyph *>::iterator iter = glyphs.begin() ; iter != glyphs.end() ; iter++) {
			(*iter)->updateStateSwitchTime((float)area.getDeltaTime());
		}

	}

	void FluidListener::stop(SXRenderArea &area) {
		delete grid;
		UniformFloat &ugroupsize = shadeX.getUniformFloat("groupsize.count");
		int groupsize = (int)ugroupsize.value;
		for(list<Glyph *>::iterator iter = glyphs.begin() ; iter != glyphs.end() ; iter++) {
			delete (*iter);
		}
	}

	void FluidListener::setMaxGlyphs(unsigned int glyphs) {
		if(GROUPS != glyphs) {
			GROUPS = glyphs;
			updateGlyphCount = true;
		}
	}

	void FluidListener::setView(ViewOption view) {
		this->view = view;
	}

}

#endif