#ifndef _FLUID_GLYPH_CPP_
#define _FLUID_GLYPH_CPP_

#include <fluid.h>
#include <sx/Log4SX.h>
#include <sstream>
using namespace std;
using namespace sx;

namespace fluid {

	float Glyph::maxStateSwitchTime = 0.25f;
	float Glyph::maxRespawnDistance = 0.05f;
	float Glyph::maxKillDistance = 0.025f;

	void Glyph::setMaxStateSwitchTime(float t) {
		maxStateSwitchTime = t;
	}

	void Glyph::setMaxRespawnDistance(float d) {
		maxRespawnDistance = d;
	}

	void Glyph::setMaxKillDistance(float d) {
		maxKillDistance = d;
	}

	Glyph::Glyph(unsigned int id) {
		this->id = id;

		RenderObject &glyph = FluidListener::shadeX.getRenderObject("glyph.object");
		unordered_map<string, Uniform *> &uniforms = glyph.getUniformSet(id);
		RenderObject &glyphshadow = FluidListener::shadeX.getRenderObject("glyphshadow.object");
		unordered_map<string, Uniform *> &shadowUniforms = glyphshadow.getUniformSet(id);

		this->state = STATE_READY;
		this->stateswitchtime = 0.0f;
		visibility = new UniformFloat("visibility");
		visibility->value = 0.0f;
		uniforms["visibility"] = visibility;
		shadowUniforms["visibility"] = visibility;
		recordslowdownpos = 0;
		slowdowntime = 0;

		vorticity = 0;

		groupsize = 13;

		for(unsigned int i=0 ; i<groupsize ; i++) {
			stringstream cp_name;
			cp_name << "controlpoint" << (i+1);
			positions[i] = new UniformVector(cp_name.str());
			uniforms[cp_name.str()] = positions[i];
			shadowUniforms[cp_name.str()] = positions[i];
			stringstream d_name;
			d_name << "direction" << (i+1);
			velocities[i] = new UniformVector(d_name.str());
			uniforms[d_name.str()] = velocities[i];
			shadowUniforms[d_name.str()] = velocities[i];
			oldpositions[i] = new Vector();
			oldvelocities[i] = new Vector();
		}
		glyphsize = new UniformFloat("glyphsize");
		uniforms["glyphsize"] = glyphsize;
		shadowUniforms["glyphsize"] = glyphsize;
	}

	Glyph::~Glyph() {
		RenderObject &glyph = FluidListener::shadeX.getRenderObject("glyph.object");
		glyph.removeUniformSet(id);
		RenderObject &glyphshadow = FluidListener::shadeX.getRenderObject("glyphshadow.object");
		glyphshadow.removeUniformSet(id);
		
		delete visibility;
		for(unsigned int i=0 ; i<groupsize ; i++) {
			delete positions[i];
			delete velocities[i];
			delete oldpositions[i];
			delete oldvelocities[i];
		}
		delete glyphsize;
	}

	unsigned int Glyph::getID() const {
		return id;
	}

	void Glyph::reset(const Vector &startpos, float *data) {
		slowdowntime = 0;
		vorticity = startpos[2];
		unsigned int midpointpos = (unsigned int)floor((float)groupsize/2.0f);
		for(unsigned int i=0 ; i<3 ; i++) {
			data[midpointpos*4 + i] = startpos[i];
		}
		data[midpointpos*4 + 3] = 0;
		for(unsigned int e=0 ; e<groupsize ; e++) {
			(*positions[e]) = startpos;
			(*oldpositions[e]) = startpos;
			(*velocities[e]) = Vector();
			(*oldvelocities[e]) = Vector();
		}
	}

	const Vector &Glyph::getMidpoint() {
		unsigned int midpointpos = (unsigned int)floor((float)groupsize/2.0f);
		return *positions[midpointpos];
	}

	void Glyph::updatePositions(const float *data) {
		float maxVorticity = 0.0f;
		for(unsigned int e=0 ; e<groupsize ; e++) {
			for(unsigned int i=0 ; i<3 ; i++) {
				(*positions[e])[i] = data[e*4 + i]*0.2f + (*oldpositions[e])[i]*0.8f;
				(*oldpositions[e])[i] = (*positions[e])[i];
			}
			//get maximum vorticity
			maxVorticity = max(maxVorticity , (*positions[e])[2] );
			(*positions[e])[2] = vorticity;

			(*positions[e])[3] = 1.0f;
		}
		//deal with vorticity: the maximum vorticity of the controlpoints
		//is used as the vorticity of the glyph
		vorticity = maxVorticity;
	}

	void Glyph::updateVelocities(const float *data) {
		for(unsigned int e=0 ; e<groupsize ; e++) {
			for(unsigned int i=0 ; i<3 ; i++) {
				(*velocities[e])[i] = data[e*4 + i]*0.2f + (*oldvelocities[e])[i]*0.8f;
				(*oldvelocities[e])[i] = (*velocities[e])[i];
			}
			(*velocities[e])[3] = 1.0f;
		}
	}

	void Glyph::updateGlyphsize() {
		glyphsize->value = (*positions[0]).distance(*positions[groupsize-1]);
		if(glyphsize->value < 0.01f) {
			//assign positive directions to glyphs
			//if necessary
			Vector v = Vector(1,0,0);
			for(unsigned int e=0 ; e<groupsize ; e++) {
				for(unsigned int i=0 ; i<4 ; i++) {
					(*velocities[e])[i] = v[i];
				}
			}
		}
	}

	unsigned int Glyph::getGroupSize() const {
		return groupsize;
	}

	Vector &Glyph::operator[](unsigned int i) {
		return *positions[i];
	}

	float Glyph::getVorticity() const {
		return vorticity;
	}

	float Glyph::length() const {
		float l = 0.0f;

		Vector *lastP = 0;
		for(unsigned int e=0 ; e<groupsize ; e++) {
			Vector *p = positions[e];
			if(lastP != 0) {
				l += ((*p) + (-1.0f)*(*lastP)).length();
			}
			lastP = p;
		}
		return l;
	}

	float Glyph::killDistance() const {
		return maxRespawnDistance * (1.0f - vorticity) + maxKillDistance * vorticity;
	}

	float Glyph::respawnDistance() const {
		return killDistance() * 2.0f;
	}

	void Glyph::setAlive() {
		state = STATE_ALIVE;
	}

	void Glyph::kill() {
		state = STATE_DEAD;
	}

	void Glyph::updateStateSwitchTime(float deltatime) {
		if(state == STATE_READY) {
			stateswitchtime = 0;
		} else if(state == STATE_ALIVE) {
			stateswitchtime = min(maxStateSwitchTime , deltatime + stateswitchtime);
		} else if(state == STATE_DEAD) {
			stateswitchtime = max(0 , stateswitchtime - deltatime);
			if(stateswitchtime == 0) {
				state = STATE_READY;
			}
		}
		if(maxStateSwitchTime > 0) {
			visibility->value = stateswitchtime / maxStateSwitchTime;
		} else {
			if(state == STATE_ALIVE) {
				visibility->value = 1.0f;
			} else {
				visibility->value = 0.0f;
			}
		}

		if(state == STATE_ALIVE) {
			//record a slowdownpos
			//this position is used to check
			//wether the glyph is moving or not
			//over time
			if(recordslowdownpos > 1) {
				recordslowdownpos = 0;
				slowdownpos = (*positions[0]);
			} else {
				recordslowdownpos += deltatime;
			}

			//glyphs, which are not moving for some time are killed
			float maxmoved = 0;
			for(unsigned int e=0 ; e<groupsize ; e++) {
				Vector pos = *positions[e];
				maxmoved = max(maxmoved , (pos + (-1.0f) * slowdownpos).length());
			}
			if(maxmoved < 0.005f && slowdowntime > 1.0f) {
				kill();
			} else if(maxmoved < 0.05f && slowdowntime <= 1.0f) {
				slowdowntime += deltatime;
			} else {
				slowdowntime = 0;
			}

			Vector pos = *positions[0];
			//glyphs outside the visible region are killed
			if(pos[0] < -1.2f || pos[0] > 1.2f || pos[1] < -1.2f || pos[1] > 1.2f) {
				kill();
			}
		}
	}

	GlyphState Glyph::getState() const {
		return state;
	}

	bool glyphComparison(Glyph *g1, Glyph *g2) {
		return g1->getVorticity() > g2->getVorticity();
	}

}

#endif